<?php
namespace AkTest\Filter;

use Ak\Filter\ArrayFilter;
use Laminas\Filter\FilterInterface;
use Laminas\Filter\FilterPluginManager;
use Laminas\Filter\ToInt;
use Laminas\ServiceManager\AbstractPluginManager;
use RuntimeException;
use stdClass;

/**
 * Class ArrayFilterTest
 *
 * @package AkTest\Filter
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/filter
 */
final class ArrayFilterTest extends AbstractTestCase
{
    public function testIsRegistered()
    {
        $container = $this->getContainer();
        $filters = $container->get(FilterPluginManager::class);

        $this->assertContainerHasService($filters, [
            'array',
            ArrayFilter::class,
        ], ArrayFilter::class, false);
    }

    public function testFilterIsPluggable()
    {
        $container = $this->getContainer();
        /** @var AbstractPluginManager $filters */
        $filters = $container->get(FilterPluginManager::class);

        /** @var ArrayFilter $filter */
        $filter = $filters->get('array');

        $this->assertSame($filters, $filter->getFilterPluginManager());
    }

    public function testCanPassInnerFilterAsConstructorArgument()
    {
        $inner = $this->getMockFilter();
        $filter = new ArrayFilter($inner);

        $this->assertSame($inner, $filter->getFilter());
    }

    public function testCanPassInnerFilterAsOption()
    {
        $container = $this->getContainer();
        /** @var FilterPluginManager $filters */
        $filters = $container->get(FilterPluginManager::class);

        $inner = $this->getMockFilter();
        $filter = $filters->get('array', [
            'filter' => $inner,
        ]);

        $this->assertSame($inner, $filter->getFilter());
    }

    public function testCanSetFilterAsString()
    {
        /** @var ArrayFilter $filter */
        $filter = $this->getFilter('array', [
            'filter' => 'toint',
        ]);

        $this->assertInstanceOf(ToInt::class, $filter->getFilter());
    }

    public function testCanSetFilterAsArray()
    {
        /** @var ArrayFilter $filter */
        $filter = $this->getFilter('array', [
            'filter' => [
                'name' => ToInt::class,
            ],
        ]);

        $this->assertInstanceOf(ToInt::class, $filter->getFilter());
    }

    public function testWillThrowExceptionIfNameNotSet()
    {
        $this->expectException(RuntimeException::class);

        /** @var ArrayFilter $array */
        $array = $this->getFilter('array', [
            'filter' => [],
        ]);

        $array->getFilter();
    }

    public function testWillThrowExceptionIfFilterNotFilter()
    {
        $this->expectException(RuntimeException::class);

        /** @var ArrayFilter $array */
        $array = $this->getFilter('array', [
            'filter' => new stdClass(),
        ]);

        $array->getFilter();
    }

    public function testWillThrowExceptionIfFilterNotSet()
    {
        $this->expectException(RuntimeException::class);

        $filter = $this->getFilter('array');
        $filter->filter([]);
    }

    public function testWillThrowExceptionIfValueNotArray()
    {
        $this->expectException(\Laminas\Filter\Exception\RuntimeException::class);

        $filter = $this->getFilter('array', [
            'filter' => $this->getMockFilter(),
        ]);

        $filter->filter('');
    }

    public function testWillFilterTheArray()
    {
        $filter = $this->getFilter('array', [
            'filter' => 'striptags',
        ]);

        $input = [
            '<h1>Naslov</h1>',
            '<p>sadržaj</p>',
        ];

        $filtered = $filter->filter($input);

        $this->assertEquals(['Naslov', 'sadržaj'], $filtered);
    }

    public function testCanUseCustomFiltersInOptions()
    {
        $filter = $this->createMock(FilterInterface::class);
        $filters = $this->getFilterManager([
            'services' => [
                'customfilter' => $filter,
            ],
        ]);

        /** @var ArrayFilter $array */
        $array = $filters->get('array', [
            'filter' => [
                'name' => 'customfilter',
            ]
        ]);

        $this->assertInstanceOf(FilterInterface::class, $array->getFilter());
    }
}
